@@ -353,6 +353,12 @@ def test_to_hdf_kwargs():
         df2 = pd.read_hdf(fn, 'foo4')
         tm.assert_frame_equal(df, df2)
 
+    # test shorthand 't' for table
+    with tmpfile('h5') as fn:
+        ddf.to_hdf(fn, 'foo4', format='t', min_itemsize=4)
+        df2 = pd.read_hdf(fn, 'foo4')
+        tm.assert_frame_equal(df, df2)
+
 
 @pytest.mark.skipif(sys.version_info[:2] == (3, 3),
                     reason="Python3.3 uses pytest2.7.2, w/o warns method")
@@ -409,6 +415,14 @@ def test_read_hdf(data, compare):
         assert (sorted(dd.read_hdf(fn, '/data', mode='r').dask) ==
                 sorted(dd.read_hdf(fn, '/data', mode='r').dask))
 
+    with tmpfile('h5') as fn:
+        sorted_data = data.sort_index()
+        sorted_data.to_hdf(fn, '/data', format='table')
+        a = dd.read_hdf(fn, '/data', chunksize=2, sorted_index=True, mode='r')
+        assert a.npartitions == 2
+
+        compare(a.compute(), sorted_data)
+
 
 def test_read_hdf_multiply_open():
     """Test that we can read from a file that's already opened elsewhere in
